-- Class talent specs
local role = {
	DEATHKNIGHT = {
		Blood         = "Melee",
		Frost         = "Melee",
		Unholy        = "Melee"
	},
	DRUID = {
		Balance       = "Ranged",
--		Feral Combat  = "Melee",
		Restoration   = "Healer"
	},
	HUNTER = {
--		Beast Mastery = "Ranged",
		Marksmanship  = "Ranged",
		Survival      = "Ranged"
	},
	MAGE = {
		Arcane        = "Ranged",
		Fire          = "Ranged",
		Frost         = "Ranged"
	},
	PALADIN = {
		Holy          = "Healer",
		Protection    = "Melee",
		Retribution   = "Melee"
	},
	PRIEST = {
		Discipline    = "Healer",
		Holy          = "Healer",
		Shadow        = "Ranged"
	},
	ROGUE = {
		Assassination = "Melee",
		Combat        = "Melee",
		Subtlety      = "Melee"
	},
	SHAMAN = {
		Elemental     = "Ranged",
		Enhancement   = "Melee",
		Restoration   = "Healer"
	},
	WARLOCK = {
		Affliction    = "Ranged",
		Demonology    = "Ranged",
		Destruction   = "Ranged"
	},
	WARRIOR = {
		Arms          = "Melee",
		Fury          = "Melee",
		Protection    = "Melee"
	}
}
role["DRUID"]["Feral Combat"] = "Melee"
role["HUNTER"]["Beast Mastery"] = "Ranged"

-- Display names
local display = {
	DEATHKNIGHT = {
		Blood         = "Blood DK",
		Frost         = "Frost DK",
		Unholy        = "Unholy DK"
	},
	DRUID = {
		Balance       = "Boomkin",
--		Feral Combat  = "Feral Druid",
		Restoration   = "Resto Druid"
	},
	HUNTER = {
--		Beast Mastery = "BM Hunter",
		Marksmanship  = "MM Hunter",
		Survival      = "Surv Hunter"
	},
	MAGE = {
		Arcane        = "Arcane Mage",
		Fire          = "Fire Mage",
		Frost         = "Frost Mage"
	},
	PALADIN = {
		Holy          = "Holy Pally",
		Protection    = "Prot Pally",
		Retribution   = "Ret Pally"
	},
	PRIEST = {
		Discipline    = "Disc Priest",
		Holy          = "Holy Priest",
		Shadow        = "Shadow Priest"
	},
	ROGUE = {
		Assassination = "Mut Rogue",
		Combat        = "Combat Rogue",
		Subtlety      = "Sub Rogue"
	},
	SHAMAN = {
		Elemental     = "Ele Sham",
		Enhancement   = "Enhance Sham",
		Restoration   = "Resto Sham"
	},
	WARLOCK = {
		Affliction    = "Aff Lock",
		Demonology    = "Demo Lock",
		Destruction   = "Destro Lock"
	},
	WARRIOR = {
		Arms          = "Arms Warr",
		Fury          = "Fury Warr",
		Protection    = "Prot Warr"
	}
}
display["DRUID"]["Feral Combat"] = "Feral Druid"
display["HUNTER"]["Beast Mastery"] = "BM Hunter"

-- Class colours
local colours = {
	DEATHKNIGHT    = "|cFFC41F3B",
	DRUID          = "|cFFFF7D0A",
	HUNTER         = "|cFFABD473",
	MAGE           = "|cFF69CCF0",
	PALADIN        = "|cFFF58CBA",
	PRIEST         = "|cFFFFFFFF",
	ROGUE          = "|cFFFFF569",
	SHAMAN         = "|cFF0070DE",
	WARLOCK        = "|cFF9482C9",
	WARRIOR        = "|cFFC79C6E"
}

local function list(side, report)
	-- Check if player is in a Battleground
	if GetNumBattlefieldScores() == 0 then
		print("BG Spy: Not in a Battleground.")
		return
	end
	
	-- Show scores for all players
	SetBattlefieldScoreFaction()
	
	-- Build list of players
	local players = {}
	local n = 0
	local total = {
		Melee  = 0,
		Ranged = 0,
		Healer = 0,
		none = 0
	}
	for i = 1, GetNumBattlefieldScores() do
		local _, _, _, _, _, faction, _, _, classToken, _, _, _, _, _, _, talentSpec = GetBattlefieldScore(i)
		if side == "Horde" and faction == 0 or side == "Alliance" and faction == 1 then
			if talentSpec then
				n = n + 1
				players[n] = {
					class = classToken,
					spec = talentSpec
				}
				total[role[classToken][talentSpec]] = total[role[classToken][talentSpec]] + 1
			else
				total.none = total.none + 1
			end
		end
	end
	if n == 0 then
		print("BG Spy: No one seen.")
		return
	end
	
	-- Sort list
	table.sort(players, function(a, b) return a.class == b.class and a.spec < b.spec or a.class < b.class end)
	
	-- Display List
	local last = display[players[1].class][players[1].spec]
	local x = 1
	local function out(i)
		if report then
			SendChatMessage(x .. "x " .. last, "BATTLEGROUND")
		else
			print(x .. "x " .. colours[players[i].class] .. last)
		end
	end
	if n > 1 then
		for i = 2, n do
			local current = display[players[i].class][players[i].spec]
			if current ~= last then
				out(i-1)
				last = current
				x = 0
			end
			x = x + 1
		end
	end
	out(n)
	
	-- Display role totals
	local message = side .. ": " .. total.Melee .. " melee, " .. total.Ranged .. " ranged, " .. total.Healer .. ( total.Healer == 1 and " healer." or " healers." ) .. ( total.none > 0 and " (" .. total.none .. " with no talents)" or "" )
	if report then
		SendChatMessage(message, "BATTLEGROUND")
	else
		print(message)
	end
end

SLASH_BGSPY1, SLASH_BGSPY2 = "/bgspy", "/bgs"
SlashCmdList.BGSPY = function(msg)
	local playerFaction = GetBattlefieldArenaFaction() == 0 and "Horde" or "Alliance"
	local enemyFaction = playerFaction == "Horde" and "Alliance" or "Horde"
	
	if msg == "" then
		list(enemyFaction)
	elseif msg == "friendly" or msg == "f" then
		list(playerFaction)
	elseif msg == "report" or msg == "r" then
		list(enemyFaction, true)
	elseif msg == "report friendly" or msg == "friendly report" or msg == "rf" or msg == "fr" then
		list(playerFaction, true)
	else
		print("BG Spy: Invalid command.")
		print("/bgspy ( /bgs )")
		print("/bgspy friendly ( /bgs f )")
		print("/bgspy report ( /bgs r )")
		print("/bgspy report friendly ( /bgs rf )")
	end
end
